macroScript MAxTRIX category:"Bobo_s Tools"
(

----------------------------------------------------------------------------------------
--   Script Name:	MAxTRIX
--	 Version:		v0.2.1
--	 Started:       04/23/2001
--	 Last Modified: 19/05/2010 - Added VRay Physcam support
--   Description:	Bullet Time Camera Rendering
--	 Documentation: Visit http://www.scriptspot.com/bobo/mxs4/maxtrix
--	 Code by:		Borislav Petrov [Boboland]		bobo@email.archlab.tuwien.ac.at
-- 	 0.2.1 modification by: Emauele Lecchi (Lele) emanuele.lecchi@gmail.com
----------------------------------------------------------------------------------------
--   NEW IN v0.2.0
--
--	 *Implemented Setup data storage in camera nodes - each camera remembers its own MAxTRIX settings. 
--	 (Cameras could be merged from scenes to bring in their MAxTRIX settings, too).
--   *Added RPF channels support
--	 *Added RPF and RLA as extension options - you can still type any supported extension manually
--	 even if it is not on the list of extensions in the File picker dialog...
----------------------------------------------------------------------------------------
--   NEW IN v0.1.2
--
--   *MAxTRIX Camera will use all animatable properties of the source camera. (FOV, Clipping etc.)
----------------------------------------------------------------------------------------
--   NEW IN v0.1.1
--
--   *Fixed a bug with Target Cameras.
----------------------------------------------------------------------------------------

global maxtrix_floater


fn leadzero num =
(
	zero = ""
	if num < 10 then zero = "000" 
	if num > 9 and num < 100 then zero = "00" 
	if num > 99 and num < 1000 then zero = "0" 
zero
)

fn create_trackview_controls =
(
	try
	(
		master_track = TrackViewNodes.MAxTRIX
	)
	catch
	(
		master_track = NewTrackViewNode "MAxTRIX"
		new_controller = addTrackViewController master_track (bezier_float()) "Speed"
		new_key = addNewKey new_controller animationrange.start
		new_key.value = 100.0
		new_key = addNewKey new_controller animationrange.end
		new_key.value = 100.0
	)
)

fn get_scene_cameras =
(
	cam_names =#()
	cams_array = for c in cameras where classof c != TargetObject collect c
	for c in cams_array do
	(
		append cam_names c.name
	)
	cam_names 
)

fn filter_camera obj = (( superclassof obj == Camera) or ( superclassof obj == VRayPhysicalCamera))

rollout maxtrix_roll "Bullet Camera Setup"
(
	group "Source Camera"
	(
		pickbutton pick_camera ">" filter:filter_camera across:3 align:#left tooltip:"Pick Camera From Scene"
		dropdownlist scene_cameras "" items:#() align:#center width:180
		button sel_camera "S" align:#right tooltip:"Select Camera In Scene"
	)
	group "Time"
	(
		radiobuttons anim_segment labels:#("Scene Segment","User Range")
		spinner anim_start "From:" range:[animationrange.start,animationrange.end,animationrange.start] type:#integer across:3 fieldwidth:40 enabled:false
		spinner anim_end "To:" range:[animationrange.start,animationrange.end,animationrange.end] type:#integer fieldwidth:40 enabled:false
		spinner anim_step "Nth:" range:[1,(animationrange.end-animationrange.start),1] type:#integer fieldwidth:40 enabled:false
	)

	group "Options"
	(
		checkbutton clamp_negative "Clamp < 0" checked:true align:#left across:2
		dropdownlist render_output items:#("Preview In Viewport Only","Save Viewport Preview","Render Final Animation") width:160 align:#right
	)

	group "Output"
	(
		button set_out "Set Output Location" width:220 
		edittext out_path text:(GetDir #image)
		edittext out_name text:"maxtrix_" across:2 align:#left fieldwidth:180
		edittext out_ext text:".tga" align:#right fieldwidth:40
	)
	
	group "RLA/RPF Options"
	(
		checkbutton channel_zDepth "Z Depth" align:#left across:4 width:56 highlightcolor:(color 200 255 200)
		checkbutton channel_matId "Mat ID" align:#center width:56 highlightcolor:(color 200 255 200)
		checkbutton channel_objectId "Object ID" align:#center width:56 highlightcolor:(color 200 255 200)
		checkbutton channel_UVCoords "UV Coords" align:#right width:56 highlightcolor:(color 200 255 200)
		checkbutton channel_Normal "Normal" align:#left across:4 width:56 highlightcolor:(color 200 255 200)
		checkbutton channel_unClamped "Unclmpd" align:#center width:56 highlightcolor:(color 200 255 200)
		checkbutton channel_Coverage "Coverage" align:#center width:56 highlightcolor:(color 200 255 200)
		checkbutton channel_Node "Node" align:#right width:56 highlightcolor:(color 200 255 200)
		checkbutton channel_ShaderColor "Sh.Color" align:#left across:4 width:56 highlightcolor:(color 200 255 200)
		checkbutton channel_ShaderTransparency "Sh.Trnsp." align:#center width:56 highlightcolor:(color 200 255 200)
		checkbutton channel_Velocity "Velocity" align:#center width:56 highlightcolor:(color 200 255 200)
		checkbutton channel_Weight "Weight" align:#right width:56 highlightcolor:(color 200 255 200)
	)

	progressbar render_progress height:8 width:240 align:#center
	button render_effect "I KNOW KUNG-FU... SHOW ME!" height:30 width:240 align:#center

	group "Note"
	(
		label note01 "Please make sure Time Display under"
		label note02 "Time Config. is set to FRAME:TICKS"
	)
	
	
	fn load_maxtrix_setup =
	(
		app_data_start = 20020106 	
		try
		(
			cam = (execute("$'"+scene_cameras.selected+"'"))
			anim_segment.state = execute ( getAppData cam (app_data_start+11) )
			anim_start.enabled = anim_end.enabled =anim_step.enabled = anim_segment.state == 2			
			anim_start.value = execute ( getAppData cam (app_data_start+12) )
			anim_end.value = execute ( getAppData cam (app_data_start+13) )
			anim_step.value = execute ( getAppData cam (app_data_start+14) )
		
			clamp_negative.state = execute ( getAppData cam (app_data_start+21) )
			render_output.selection = execute ( getAppData cam (app_data_start+22) )

			out_path.text = getAppData cam (app_data_start+31)
			out_name.text = getAppData cam (app_data_start+32) 
			out_ext.text = getAppData cam (app_data_start+33)
			channel_zDepth.state = execute ( getAppData cam (app_data_start+41) )	
			channel_matId.state = execute ( getAppData cam (app_data_start+42) )
			channel_objectId.state = execute ( getAppData cam (app_data_start+43) )
			channel_UVCoords.state = execute ( getAppData cam (app_data_start+44) )
			channel_Normal.state = execute ( getAppData cam (app_data_start+45) )			
			channel_unClamped.state = execute ( getAppData cam (app_data_start+46) )
			channel_Coverage.state = execute ( getAppData cam (app_data_start+47) )
			channel_Node.state = execute ( getAppData cam (app_data_start+48) )
			channel_ShaderColor.state = execute ( getAppData cam (app_data_start+49) )
			channel_ShaderTransparency.state = execute ( getAppData cam (app_data_start+50) )
			channel_Velocity.state = execute ( getAppData cam (app_data_start+51) )
			channel_Weight.state = execute ( getAppData cam (app_data_start+52) )	
		)
		catch()
	)

	fn save_maxtrix_setup =
	(
		app_data_start = 20020106
		try
		(
			cam = (execute("$'"+scene_cameras.selected+"'"))
			setAppData cam (app_data_start+11) (anim_segment.state as string)
			setAppData cam (app_data_start+12) (anim_start.value as string)
			setAppData cam (app_data_start+13) (anim_end.value as string)
			setAppData cam (app_data_start+14) (anim_step.value as string)
		
			setAppData cam (app_data_start+21) (clamp_negative.state as string)
			setAppData cam (app_data_start+22) (render_output.selection as string)

			setAppData cam (app_data_start+31) (out_path.text)
			setAppData cam (app_data_start+32) (out_name.text)
			setAppData cam (app_data_start+33) (out_ext.text)
			setAppData cam (app_data_start+41) (channel_zDepth.state as string)			
			setAppData cam (app_data_start+42) (channel_matId.state as string)			
			setAppData cam (app_data_start+43) (channel_objectId.state as string)			
			setAppData cam (app_data_start+44) (channel_UVCoords.state as string)			
			setAppData cam (app_data_start+45) (channel_Normal.state as string)			
			setAppData cam (app_data_start+46) (channel_unClamped.state as string)			
			setAppData cam (app_data_start+47) (channel_Coverage.state as string)			
			setAppData cam (app_data_start+48) (channel_Node.state as string)			
			setAppData cam (app_data_start+49) (channel_ShaderColor.state as string)			
			setAppData cam (app_data_start+50) (channel_ShaderTransparency.state as string)			
			setAppData cam (app_data_start+51) (channel_Velocity.state as string)			
			setAppData cam (app_data_start+52) (channel_Weight.state as string)			
		)catch()	
	)	
	
	fn build_request_channels =
	(
		ch_array = #()
		if channel_zDepth.checked then append ch_array #zDepth
		if channel_matId.checked then append ch_array #matId
		if channel_objectId.checked then append ch_array #objectId
		if channel_UVCoords.checked then append ch_array #UVCoords
		if channel_Normal.checked then append ch_array #Normal
		if channel_unClamped.checked then append ch_array #unClamped
		if channel_Coverage.checked then append ch_array #Coverage
		if channel_Node.checked then append ch_array #Node
		if channel_ShaderColor.checked then append ch_array #ShaderColor
		if channel_ShaderTransparency.checked then append ch_array #ShaderTransparency
		if channel_Velocity.checked then append ch_array #Velocity
		if channel_Weight.checked then append ch_array #Weight
	ch_array
	)

	on anim_segment changed state do
	(
		anim_start.enabled = anim_end.enabled =anim_step.enabled = state == 2
		save_maxtrix_setup()
	)
	on anim_start changed val do save_maxtrix_setup()
	on anim_end changed val do save_maxtrix_setup()
	on anim_step changed val do save_maxtrix_setup()
	on clamp_negative changed state do save_maxtrix_setup()
	on render_output selected item do save_maxtrix_setup()
	on out_path entered txt do save_maxtrix_setup()
	on out_name entered txt do save_maxtrix_setup()
	on out_ext entered txt do save_maxtrix_setup()
	
	on channel_zDepth changed state do save_maxtrix_setup()
	on channel_matId changed state do save_maxtrix_setup()
	on channel_objectId changed state do save_maxtrix_setup()
	on channel_UVCoords changed state do save_maxtrix_setup()
	on channel_Normal changed state do save_maxtrix_setup()
	on channel_unClamped changed state do save_maxtrix_setup()
	on channel_Coverage changed state do save_maxtrix_setup()
	on channel_Node changed state do save_maxtrix_setup()
	on channel_ShaderColor changed state do save_maxtrix_setup()
	on channel_ShaderTransparency changed state do save_maxtrix_setup()
	on channel_Velocity changed state do save_maxtrix_setup()
	on channel_Weight changed state do save_maxtrix_setup()
	

	
	on set_out pressed do
	(
		new_out_path = GetSaveFileName types:"Targa (*.tga)|*.tga|Rich Pixel Format (*.rpf)|*.rpf|RLA (*.rla)|*.rla|JPEG (*.jpg)|*.jpg|BMP (*.bmp)|*.bmp|TIFF (*.tif)|*.tif"
		if new_out_path != undefined then
		(
			out_path.text = getfilenamepath new_out_path 
			out_name.text = getfilenamefile new_out_path 
			out_ext.text = getfilenametype new_out_path 
			save_maxtrix_setup()
		)
	)

	on sel_camera pressed do try( select (execute("$'"+scene_cameras.selected+"'")))catch()

	on pick_camera picked obj do 
	(
		source_camera = obj
		get_scene_cameras ()
		scene_cameras.selection = finditem scene_cameras.items obj.name
		load_maxtrix_setup()
	)
	
	on scene_cameras selected itm do 
	(
		load_maxtrix_setup()
	)	

	on render_effect pressed do
	(
		go_on = false
		try
		(
			source_camera = (execute("$'"+scene_cameras.selected+"'"))
			go_on = true
		)
		catch
		(
		)
		if go_on then
		(
			new_cam = Freecamera()
			new_cam.name = uniquename "MAxTRIX_Temp"
			viewport.SetCamera new_cam
			if render_output.selection == 3 then
			(
				bmp_view = bitmap renderwidth renderheight
				display bmp_view
			)
			case anim_segment.state of
			(
			1: (
				start_time = ((animationrange.start as float)/TicksPerFrame) as integer
				end_time = ((animationrange.end as float)/TicksPerFrame) as integer
				step_time = 1
				)
			2: (
				start_time = anim_start.value
				end_time = anim_end.value
				step_time = anim_step.value
				)	
			)
	
			--world_time = animationrange.start
			world_time = start_time as time
			render_frame = false
			counter = (step_time-1)
			for i = start_time to end_time do
			(
				render_progress.value = 100.0*(i-start_time)/(end_time-start_time) 
				counter += 1
				if counter == step_time then 
				(
					counter = 0
					render_frame = true
				)
				at time i slow_time = TrackViewNodes.maXtrix.Speed.value
				if slow_time < 0 and clamp_negative.checked then slow_time = 0
				at time i 
				(
					new_cam.transform = source_camera.transform
					cam_props = getpropnames source_camera
					for p = 2 to cam_props.count do
					(
						try 
						(
							setproperty new_cam (cam_Props[p]) (getproperty source_camera (cam_Props[p]))
						)catch()
					)
				)
				world_time += 1f * slow_time/100.0
				if render_frame then 
				(
					slidertime = world_time 
					if render_output.selection == 2 then
					(
						bmp_out = gw.getviewportDib()
						bmp_out.filename = (out_path.text+"\\"+out_name.text+(leadzero i) + i as string +out_ext.text)
						save bmp_out
						close bmp_out
					)--end if
					if render_output.selection == 3 then
					(
						bmp_out = bitmap renderwidth renderheight
						bmp_out.filename = (out_path.text+"\\"+out_name.text+(leadzero i) + i as string +out_ext.text)
						request_channels = build_request_channels()
						render to:bmp_view channels:request_channels
						copy bmp_view bmp_out
						save bmp_out
						close bmp_out
						gc()
					)--end if
					render_frame = false
				)--end if render_frame
			)--end loop
			render_progress.value = 0
			bmp_out = undefined
			delete $MAxTRIX_Temp*
		)--end if go_on
	)--end on
)--end rollout

create_trackview_controls()
try(closerolloutfloater maxtrix_floater)catch()
maxtrix_floater = newrolloutfloater "MAxTRIX 0.2" 280 545 100 100
addrollout maxtrix_roll maxtrix_floater 

maxtrix_roll.scene_cameras.items = get_scene_cameras()
if maxtrix_roll.scene_cameras.items.count < 1 then 
(
	maxtrix_roll.render_effect.enabled = false
)
else
(
	maxtrix_roll.load_maxtrix_setup()
)

)--end script